@@ -3,6 +3,7 @@ import re
 from unittest.mock import Mock
 
 from sqlalchemy import Column
+from sqlalchemy import create_engine
 from sqlalchemy import event
 from sqlalchemy import exc
 from sqlalchemy import inspect
@@ -423,7 +424,7 @@ class FastExecutemanyTest(fixtures.TestBase):
     @testing.variation("add_event", [True, False])
     @testing.variation("setinputsizes", [True, False])
     @testing.variation("fastexecutemany", [True, False])
-    @testing.variation("insertmanyvalues", [True, False])
+    @testing.variation("insertmanyvalues", [False])  # disabled due to #9603
     @testing.variation("broken_types", [True, False])
     def test_insert_typing(
         self,
@@ -459,7 +460,10 @@ class FastExecutemanyTest(fixtures.TestBase):
         # (... four months pass ...)
         # surprise! we need it again.  woop!  for #8917
         expect_failure = (
-            broken_types and not setinputsizes and insertmanyvalues
+            broken_types
+            and not setinputsizes
+            and insertmanyvalues
+            and not fastexecutemany
         )
 
         engine = testing_engine(
@@ -625,6 +629,14 @@ class MiscTest(fixtures.TestBase):
     __only_on__ = "mssql"
     __backend__ = True
 
+    def test_no_insertmanyvalues(self):
+        with expect_raises_message(
+            exc.ArgumentError,
+            "The use_insertmanyvalues feature on SQL Server is "
+            "currently not safe to use",
+        ):
+            create_engine("mssql+pyodbc://", use_insertmanyvalues=True)
+
     @testing.variation("enable_comments", [True, False])
     def test_comments_enabled_disabled(
         self, testing_engine, metadata, enable_comments
