@@ -1,7 +1,11 @@
+from unittest import mock
+from unittest.mock import Mock
+
 import sqlalchemy as tsa
 from sqlalchemy import create_engine
+from sqlalchemy import create_mock_engine
 from sqlalchemy import event
-from sqlalchemy import exc
+from sqlalchemy import Index
 from sqlalchemy import Integer
 from sqlalchemy import MetaData
 from sqlalchemy import String
@@ -11,18 +15,20 @@ from sqlalchemy.schema import AddConstraint
 from sqlalchemy.schema import CheckConstraint
 from sqlalchemy.schema import DDL
 from sqlalchemy.schema import DropConstraint
-from sqlalchemy.testing import assert_raises
+from sqlalchemy.schema import ForeignKeyConstraint
+from sqlalchemy.schema import Sequence
 from sqlalchemy.testing import AssertsCompiledSQL
+from sqlalchemy.testing import config
 from sqlalchemy.testing import engines
 from sqlalchemy.testing import eq_
 from sqlalchemy.testing import fixtures
-from sqlalchemy.testing import mock
+from sqlalchemy.testing.provision import normalize_sequence
 from sqlalchemy.testing.schema import Column
 from sqlalchemy.testing.schema import Table
 
 
 class DDLEventTest(fixtures.TestBase):
-    def setup(self):
+    def setup_test(self):
         self.bind = engines.mock_engine()
         self.metadata = MetaData()
         self.table = Table("t", self.metadata, Column("id", Integer))
@@ -373,27 +379,227 @@ class DDLEventTest(fixtures.TestBase):
         )
         eq_(metadata_canary.mock_calls, [])
 
-    def test_append_listener(self):
-        metadata, table, bind = self.metadata, self.table, self.bind
 
-        def fn(*a):
-            return None
+class DDLEventHarness:
+    creates_implicitly_with_table = True
+    drops_implicitly_with_table = True
 
-        table.append_ddl_listener("before-create", fn)
-        assert_raises(
-            exc.InvalidRequestError, table.append_ddl_listener, "blah", fn
+    @testing.fixture
+    def produce_subject(self):
+        raise NotImplementedError()
+
+    @testing.fixture
+    def produce_event_target(self, produce_subject, connection):
+        """subclasses may want to override this for cases where the target
+        sent to the event is not the same object as that which was
+        listened on.
+
+        the example here is for :class:`.SchemaType` objects like
+        :class:`.Enum` that produce a dialect-specific implementation
+        which is where the actual CREATE/DROP happens.
+
+        """
+        return produce_subject
+
+    @testing.fixture
+    def produce_table_integrated_subject(self, metadata, produce_subject):
+        raise NotImplementedError()
+
+    def test_table_integrated(
+        self,
+        metadata,
+        connection,
+        produce_subject,
+        produce_table_integrated_subject,
+        produce_event_target,
+    ):
+        subject = produce_subject
+        assert_subject = produce_event_target
+
+        canary = mock.Mock()
+        event.listen(subject, "before_create", canary.before_create)
+        event.listen(subject, "after_create", canary.after_create)
+        event.listen(subject, "before_drop", canary.before_drop)
+        event.listen(subject, "after_drop", canary.after_drop)
+
+        metadata.create_all(connection, checkfirst=False)
+
+        if self.creates_implicitly_with_table:
+            create_calls = []
+        else:
+            create_calls = [
+                mock.call.before_create(
+                    assert_subject,
+                    connection,
+                    _ddl_runner=mock.ANY,
+                ),
+                mock.call.after_create(
+                    assert_subject,
+                    connection,
+                    _ddl_runner=mock.ANY,
+                ),
+            ]
+        eq_(canary.mock_calls, create_calls)
+        metadata.drop_all(connection, checkfirst=False)
+
+        if self.drops_implicitly_with_table:
+            eq_(canary.mock_calls, create_calls + [])
+        else:
+            eq_(
+                canary.mock_calls,
+                create_calls
+                + [
+                    mock.call.before_drop(
+                        assert_subject,
+                        connection,
+                        _ddl_runner=mock.ANY,
+                    ),
+                    mock.call.after_drop(
+                        assert_subject,
+                        connection,
+                        _ddl_runner=mock.ANY,
+                    ),
+                ],
+            )
+
+
+class DDLEventWCreateHarness(DDLEventHarness):
+
+    requires_table_to_exist = True
+
+    def test_straight_create_drop(
+        self,
+        metadata,
+        connection,
+        produce_subject,
+        produce_table_integrated_subject,
+        produce_event_target,
+    ):
+        subject = produce_subject
+        assert_subject = produce_event_target
+
+        if self.requires_table_to_exist:
+            metadata.create_all(connection, checkfirst=False)
+            subject.drop(connection)
+
+        canary = mock.Mock()
+        event.listen(subject, "before_create", canary.before_create)
+        event.listen(subject, "after_create", canary.after_create)
+        event.listen(subject, "before_drop", canary.before_drop)
+        event.listen(subject, "after_drop", canary.after_drop)
+
+        subject.create(connection)
+
+        eq_(
+            canary.mock_calls,
+            [
+                mock.call.before_create(
+                    assert_subject,
+                    connection,
+                    _ddl_runner=mock.ANY,
+                ),
+                mock.call.after_create(
+                    assert_subject,
+                    connection,
+                    _ddl_runner=mock.ANY,
+                ),
+            ],
         )
 
-        metadata.append_ddl_listener("before-create", fn)
-        assert_raises(
-            exc.InvalidRequestError, metadata.append_ddl_listener, "blah", fn
+        subject.drop(connection)
+
+        eq_(
+            canary.mock_calls,
+            [
+                mock.call.before_create(
+                    assert_subject,
+                    connection,
+                    _ddl_runner=mock.ANY,
+                ),
+                mock.call.after_create(
+                    assert_subject,
+                    connection,
+                    _ddl_runner=mock.ANY,
+                ),
+                mock.call.before_drop(
+                    assert_subject,
+                    connection,
+                    _ddl_runner=mock.ANY,
+                ),
+                mock.call.after_drop(
+                    assert_subject,
+                    connection,
+                    _ddl_runner=mock.ANY,
+                ),
+            ],
         )
 
 
-class DDLExecutionTest(fixtures.TestBase):
-    def setup(self):
+class SequenceDDLEventTest(DDLEventWCreateHarness, fixtures.TestBase):
+    __requires__ = ("sequences",)
+
+    creates_implicitly_with_table = False
+    drops_implicitly_with_table = False
+    supports_standalone_create = True
+
+    @testing.fixture
+    def produce_subject(self):
+        return normalize_sequence(config, Sequence("my_seq"))
+
+    @testing.fixture
+    def produce_table_integrated_subject(self, metadata, produce_subject):
+        return Table(
+            "t",
+            metadata,
+            Column("id", Integer, produce_subject, primary_key=True),
+        )
+
+
+class IndexDDLEventTest(DDLEventWCreateHarness, fixtures.TestBase):
+    creates_implicitly_with_table = False
+    drops_implicitly_with_table = True
+    supports_standalone_create = False
+
+    @testing.fixture
+    def produce_subject(self):
+        return Index("my_idx", "key")
+
+    @testing.fixture
+    def produce_table_integrated_subject(self, metadata, produce_subject):
+        return Table(
+            "t",
+            metadata,
+            Column("id", Integer, primary_key=True),
+            Column("key", String(50)),
+            produce_subject,
+        )
+
+
+class ForeignKeyConstraintDDLEventTest(DDLEventHarness, fixtures.TestBase):
+    creates_implicitly_with_table = True
+    drops_implicitly_with_table = True
+    supports_standalone_create = False
+
+    @testing.fixture
+    def produce_subject(self):
+        return ForeignKeyConstraint(["related_id"], ["related.id"], name="fkc")
+
+    @testing.fixture
+    def produce_table_integrated_subject(self, metadata, produce_subject):
+        Table(
+            "t",
+            metadata,
+            Column("id", Integer, primary_key=True),
+            Column("related_id", Integer),
+            produce_subject,
+        )
+        Table("related", metadata, Column("id", Integer, primary_key=True))
+
+
+class DDLExecutionTest(AssertsCompiledSQL, fixtures.TestBase):
+    def setup_test(self):
         self.engine = engines.mock_engine()
-        self.metadata = MetaData(self.engine)
+        self.metadata = MetaData()
         self.users = Table(
             "users",
             self.metadata,
@@ -408,14 +614,14 @@ class DDLExecutionTest(fixtures.TestBase):
         event.listen(users, "before_drop", DDL("xyzzy"))
         event.listen(users, "after_drop", DDL("fnord"))
 
-        users.create()
+        users.create(self.engine)
         strings = [str(x) for x in engine.mock]
         assert "mxyzptlk" in strings
         assert "klptzyxm" in strings
         assert "xyzzy" not in strings
         assert "fnord" not in strings
         del engine.mock[:]
-        users.drop()
+        users.drop(self.engine)
         strings = [str(x) for x in engine.mock]
         assert "mxyzptlk" not in strings
         assert "klptzyxm" not in strings
@@ -430,102 +636,20 @@ class DDLExecutionTest(fixtures.TestBase):
         event.listen(users, "before_drop", DDL("xyzzy"))
         event.listen(users, "after_drop", DDL("fnord"))
 
-        metadata.create_all()
+        metadata.create_all(self.engine)
         strings = [str(x) for x in engine.mock]
         assert "mxyzptlk" in strings
         assert "klptzyxm" in strings
         assert "xyzzy" not in strings
         assert "fnord" not in strings
         del engine.mock[:]
-        metadata.drop_all()
+        metadata.drop_all(self.engine)
         strings = [str(x) for x in engine.mock]
         assert "mxyzptlk" not in strings
         assert "klptzyxm" not in strings
         assert "xyzzy" in strings
         assert "fnord" in strings
 
-    @testing.uses_deprecated(r"See DDLEvents")
-    def test_table_by_metadata_deprecated(self):
-        metadata, users, engine = self.metadata, self.users, self.engine
-        DDL("mxyzptlk").execute_at("before-create", users)
-        DDL("klptzyxm").execute_at("after-create", users)
-        DDL("xyzzy").execute_at("before-drop", users)
-        DDL("fnord").execute_at("after-drop", users)
-
-        metadata.create_all()
-        strings = [str(x) for x in engine.mock]
-        assert "mxyzptlk" in strings
-        assert "klptzyxm" in strings
-        assert "xyzzy" not in strings
-        assert "fnord" not in strings
-        del engine.mock[:]
-        metadata.drop_all()
-        strings = [str(x) for x in engine.mock]
-        assert "mxyzptlk" not in strings
-        assert "klptzyxm" not in strings
-        assert "xyzzy" in strings
-        assert "fnord" in strings
-
-    def test_deprecated_append_ddl_listener_table(self):
-        metadata, users, engine = self.metadata, self.users, self.engine
-        canary = []
-        users.append_ddl_listener(
-            "before-create", lambda e, t, b: canary.append("mxyzptlk")
-        )
-        users.append_ddl_listener(
-            "after-create", lambda e, t, b: canary.append("klptzyxm")
-        )
-        users.append_ddl_listener(
-            "before-drop", lambda e, t, b: canary.append("xyzzy")
-        )
-        users.append_ddl_listener(
-            "after-drop", lambda e, t, b: canary.append("fnord")
-        )
-
-        metadata.create_all()
-        assert "mxyzptlk" in canary
-        assert "klptzyxm" in canary
-        assert "xyzzy" not in canary
-        assert "fnord" not in canary
-        del engine.mock[:]
-        canary[:] = []
-        metadata.drop_all()
-        assert "mxyzptlk" not in canary
-        assert "klptzyxm" not in canary
-        assert "xyzzy" in canary
-        assert "fnord" in canary
-
-    def test_deprecated_append_ddl_listener_metadata(self):
-        metadata, users, engine = self.metadata, self.users, self.engine
-        canary = []
-        metadata.append_ddl_listener(
-            "before-create",
-            lambda e, t, b, tables=None: canary.append("mxyzptlk"),
-        )
-        metadata.append_ddl_listener(
-            "after-create",
-            lambda e, t, b, tables=None: canary.append("klptzyxm"),
-        )
-        metadata.append_ddl_listener(
-            "before-drop", lambda e, t, b, tables=None: canary.append("xyzzy")
-        )
-        metadata.append_ddl_listener(
-            "after-drop", lambda e, t, b, tables=None: canary.append("fnord")
-        )
-
-        metadata.create_all()
-        assert "mxyzptlk" in canary
-        assert "klptzyxm" in canary
-        assert "xyzzy" not in canary
-        assert "fnord" not in canary
-        del engine.mock[:]
-        canary[:] = []
-        metadata.drop_all()
-        assert "mxyzptlk" not in canary
-        assert "klptzyxm" not in canary
-        assert "xyzzy" in canary
-        assert "fnord" in canary
-
     def test_metadata(self):
         metadata, engine = self.metadata, self.engine
 
@@ -534,37 +658,14 @@ class DDLExecutionTest(fixtures.TestBase):
         event.listen(metadata, "before_drop", DDL("xyzzy"))
         event.listen(metadata, "after_drop", DDL("fnord"))
 
-        metadata.create_all()
+        metadata.create_all(self.engine)
         strings = [str(x) for x in engine.mock]
         assert "mxyzptlk" in strings
         assert "klptzyxm" in strings
         assert "xyzzy" not in strings
         assert "fnord" not in strings
         del engine.mock[:]
-        metadata.drop_all()
-        strings = [str(x) for x in engine.mock]
-        assert "mxyzptlk" not in strings
-        assert "klptzyxm" not in strings
-        assert "xyzzy" in strings
-        assert "fnord" in strings
-
-    @testing.uses_deprecated(r"See DDLEvents")
-    def test_metadata_deprecated(self):
-        metadata, engine = self.metadata, self.engine
-
-        DDL("mxyzptlk").execute_at("before-create", metadata)
-        DDL("klptzyxm").execute_at("after-create", metadata)
-        DDL("xyzzy").execute_at("before-drop", metadata)
-        DDL("fnord").execute_at("after-drop", metadata)
-
-        metadata.create_all()
-        strings = [str(x) for x in engine.mock]
-        assert "mxyzptlk" in strings
-        assert "klptzyxm" in strings
-        assert "xyzzy" not in strings
-        assert "fnord" not in strings
-        del engine.mock[:]
-        metadata.drop_all()
+        metadata.drop_all(self.engine)
         strings = [str(x) for x in engine.mock]
         assert "mxyzptlk" not in strings
         assert "klptzyxm" not in strings
@@ -572,7 +673,7 @@ class DDLExecutionTest(fixtures.TestBase):
         assert "fnord" in strings
 
     def test_conditional_constraint(self):
-        metadata, users, engine = self.metadata, self.users, self.engine
+        metadata, users = self.metadata, self.users
         nonpg_mock = engines.mock_engine(dialect_name="sqlite")
         pg_mock = engines.mock_engine(dialect_name="postgresql")
         constraint = CheckConstraint(
@@ -607,69 +708,130 @@ class DDLExecutionTest(fixtures.TestBase):
         strings = " ".join(str(x) for x in pg_mock.mock)
         assert "my_test_constraint" in strings
 
-    @testing.uses_deprecated(r"See DDLEvents")
-    def test_conditional_constraint_deprecated(self):
-        metadata, users, engine = self.metadata, self.users, self.engine
+    @testing.combinations(("dialect",), ("callable",), ("callable_w_state",))
+    def test_inline_ddl_if_dialect_name(self, ddl_if_type):
         nonpg_mock = engines.mock_engine(dialect_name="sqlite")
         pg_mock = engines.mock_engine(dialect_name="postgresql")
-        constraint = CheckConstraint(
-            "a < b", name="my_test_constraint", table=users
+
+        metadata = MetaData()
+
+        capture_mock = Mock()
+        state = object()
+
+        if ddl_if_type == "dialect":
+            ddl_kwargs = dict(dialect="postgresql")
+        elif ddl_if_type == "callable":
+
+            def is_pg(ddl, target, bind, **kw):
+                capture_mock.is_pg(ddl, target, bind, **kw)
+                return kw["dialect"].name == "postgresql"
+
+            ddl_kwargs = dict(callable_=is_pg)
+        elif ddl_if_type == "callable_w_state":
+
+            def is_pg(ddl, target, bind, **kw):
+                capture_mock.is_pg(ddl, target, bind, **kw)
+                return kw["dialect"].name == "postgresql"
+
+            ddl_kwargs = dict(callable_=is_pg, state=state)
+        else:
+            assert False
+
+        data_col = Column("data", String)
+        t = Table(
+            "a",
+            metadata,
+            Column("id", Integer, primary_key=True),
+            Column("num", Integer),
+            data_col,
+            Index("my_pg_index", data_col).ddl_if(**ddl_kwargs),
+            CheckConstraint("num > 5").ddl_if(**ddl_kwargs),
         )
 
-        # by placing the constraint in an Add/Drop construct, the
-        # 'inline_ddl' flag is set to False
+        metadata.create_all(nonpg_mock)
+        eq_(len(nonpg_mock.mock), 1)
+        self.assert_compile(
+            nonpg_mock.mock[0],
+            "CREATE TABLE a (id INTEGER NOT NULL, num INTEGER, "
+            "data VARCHAR, PRIMARY KEY (id))",
+            dialect=nonpg_mock.dialect,
+        )
 
-        AddConstraint(constraint, on="postgresql").execute_at(
-            "after-create", users
+        metadata.create_all(pg_mock)
+
+        eq_(len(pg_mock.mock), 2)
+
+        self.assert_compile(
+            pg_mock.mock[0],
+            "CREATE TABLE a (id SERIAL NOT NULL, num INTEGER, "
+            "data VARCHAR, PRIMARY KEY (id), CHECK (num > 5))",
+            dialect=pg_mock.dialect,
         )
-        DropConstraint(constraint, on="postgresql").execute_at(
-            "before-drop", users
+        self.assert_compile(
+            pg_mock.mock[1],
+            "CREATE INDEX my_pg_index ON a (data)",
+            dialect="postgresql",
         )
-        metadata.create_all(bind=nonpg_mock)
-        strings = " ".join(str(x) for x in nonpg_mock.mock)
-        assert "my_test_constraint" not in strings
-        metadata.drop_all(bind=nonpg_mock)
-        strings = " ".join(str(x) for x in nonpg_mock.mock)
-        assert "my_test_constraint" not in strings
-        metadata.create_all(bind=pg_mock)
-        strings = " ".join(str(x) for x in pg_mock.mock)
-        assert "my_test_constraint" in strings
-        metadata.drop_all(bind=pg_mock)
-        strings = " ".join(str(x) for x in pg_mock.mock)
-        assert "my_test_constraint" in strings
+
+        the_index = list(t.indexes)[0]
+        the_constraint = list(
+            c for c in t.constraints if isinstance(c, CheckConstraint)
+        )[0]
+
+        if ddl_if_type in ("callable", "callable_w_state"):
+
+            if ddl_if_type == "callable":
+                check_state = None
+            else:
+                check_state = state
+
+            eq_(
+                capture_mock.mock_calls,
+                [
+                    mock.call.is_pg(
+                        mock.ANY,
+                        the_index,
+                        mock.ANY,
+                        state=check_state,
+                        dialect=nonpg_mock.dialect,
+                        compiler=None,
+                    ),
+                    mock.call.is_pg(
+                        mock.ANY,
+                        the_constraint,
+                        None,
+                        state=check_state,
+                        dialect=nonpg_mock.dialect,
+                        compiler=mock.ANY,
+                    ),
+                    mock.call.is_pg(
+                        mock.ANY,
+                        the_index,
+                        mock.ANY,
+                        state=check_state,
+                        dialect=pg_mock.dialect,
+                        compiler=None,
+                    ),
+                    mock.call.is_pg(
+                        mock.ANY,
+                        the_constraint,
+                        None,
+                        state=check_state,
+                        dialect=pg_mock.dialect,
+                        compiler=mock.ANY,
+                    ),
+                ],
+            )
 
     @testing.requires.sqlite
     def test_ddl_execute(self):
         engine = create_engine("sqlite:///")
         cx = engine.connect()
-        table = self.users
+        cx.begin()
         ddl = DDL("SELECT 1")
 
-        for py in (
-            "engine.execute(ddl)",
-            "engine.execute(ddl, table)",
-            "cx.execute(ddl)",
-            "cx.execute(ddl, table)",
-            "ddl.execute(engine)",
-            "ddl.execute(engine, table)",
-            "ddl.execute(cx)",
-            "ddl.execute(cx, table)",
-        ):
-            r = eval(py)
-            assert list(r) == [(1,)], py
-
-        for py in ("ddl.execute()", "ddl.execute(target=table)"):
-            try:
-                r = eval(py)
-                assert False
-            except tsa.exc.UnboundExecutionError:
-                pass
-
-        for bind in engine, cx:
-            ddl.bind = bind
-            for py in ("ddl.execute()", "ddl.execute(target=table)"):
-                r = eval(py)
-                assert list(r) == [(1,)], py
+        r = cx.execute(ddl)
+        eq_(list(r), [(1,)])
 
     def test_platform_escape(self):
         """test the escaping of % characters in the DDL construct."""
@@ -700,14 +862,102 @@ class DDLExecutionTest(fixtures.TestBase):
             )
 
 
+class DDLTransactionTest(fixtures.TestBase):
+    """test DDL transactional behavior as of SQLAlchemy 1.4."""
+
+    @testing.fixture
+    def metadata_fixture(self):
+        m = MetaData()
+        Table("t1", m, Column("q", Integer))
+        Table("t2", m, Column("q", Integer))
+
+        try:
+            yield m
+        finally:
+            m.drop_all(testing.db)
+
+    @testing.fixture
+    def listening_engine_fixture(self):
+        eng = engines.testing_engine()
+
+        m1 = mock.Mock()
+
+        event.listen(eng, "begin", m1.begin)
+        event.listen(eng, "commit", m1.commit)
+        event.listen(eng, "rollback", m1.rollback)
+
+        @event.listens_for(eng, "before_cursor_execute")
+        def before_cursor_execute(
+            conn, cursor, statement, parameters, context, executemany
+        ):
+            if "CREATE TABLE" in statement:
+                m1.cursor_execute("CREATE TABLE ...")
+
+        eng.connect().close()
+
+        return eng, m1
+
+    def test_ddl_engine(self, metadata_fixture, listening_engine_fixture):
+        eng, m1 = listening_engine_fixture
+
+        metadata_fixture.create_all(eng)
+
+        eq_(
+            m1.mock_calls,
+            [
+                mock.call.begin(mock.ANY),
+                mock.call.cursor_execute("CREATE TABLE ..."),
+                mock.call.cursor_execute("CREATE TABLE ..."),
+                mock.call.commit(mock.ANY),
+            ],
+        )
+
+    def test_ddl_connection_autobegin_transaction(
+        self, metadata_fixture, listening_engine_fixture
+    ):
+        eng, m1 = listening_engine_fixture
+
+        with eng.connect() as conn:
+            metadata_fixture.create_all(conn)
+
+            conn.commit()
+
+        eq_(
+            m1.mock_calls,
+            [
+                mock.call.begin(mock.ANY),
+                mock.call.cursor_execute("CREATE TABLE ..."),
+                mock.call.cursor_execute("CREATE TABLE ..."),
+                mock.call.commit(mock.ANY),
+            ],
+        )
+
+    def test_ddl_connection_explicit_begin_transaction(
+        self, metadata_fixture, listening_engine_fixture
+    ):
+        eng, m1 = listening_engine_fixture
+
+        with eng.connect() as conn:
+            with conn.begin():
+                metadata_fixture.create_all(conn)
+
+        eq_(
+            m1.mock_calls,
+            [
+                mock.call.begin(mock.ANY),
+                mock.call.cursor_execute("CREATE TABLE ..."),
+                mock.call.cursor_execute("CREATE TABLE ..."),
+                mock.call.commit(mock.ANY),
+            ],
+        )
+
+
 class DDLTest(fixtures.TestBase, AssertsCompiledSQL):
     def mock_engine(self):
         def executor(*a, **kw):
             return None
 
-        engine = create_engine(
-            testing.db.name + "://", strategy="mock", executor=executor
-        )
+        engine = create_mock_engine(testing.db.name + "://", executor)
         # fmt: off
         engine.dialect.identifier_preparer = \
             tsa.sql.compiler.IdentifierPreparer(
@@ -779,27 +1029,3 @@ class DDLTest(fixtures.TestBase, AssertsCompiledSQL):
             )
             ._should_execute(tbl, cx)
         )
-
-    @testing.uses_deprecated(r"See DDLEvents")
-    def test_filter_deprecated(self):
-        cx = self.mock_engine()
-
-        tbl = Table("t", MetaData(), Column("id", Integer))
-        target = cx.name
-
-        assert DDL("")._should_execute_deprecated("x", tbl, cx)
-        assert DDL("", on=target)._should_execute_deprecated("x", tbl, cx)
-        assert not DDL("", on="bogus")._should_execute_deprecated("x", tbl, cx)
-        assert DDL("", on=lambda d, x, y, z: True)._should_execute_deprecated(
-            "x", tbl, cx
-        )
-        assert DDL(
-            "", on=lambda d, x, y, z: z.engine.name != "bogus"
-        )._should_execute_deprecated("x", tbl, cx)
-
-    def test_repr(self):
-        assert repr(DDL("s"))
-        assert repr(DDL("s", on="engine"))
-        assert repr(DDL("s", on=lambda x: 1))
-        assert repr(DDL("s", context={"a": 1}))
-        assert repr(DDL("s", on="engine", context={"a": 1}))
